import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.util.List;

/**
 * A bullet that can hit asteroids.
 * 
 * @author Poul Henriksen
 */
public class Bullet extends MovingThing
{
    /** A bullet looses one life each act, and will disappear when life = 0 */
    private int life = 30;
    /** The damage this bullet will deal */
    private int damage = 16;
    private Rocket rocket;

    public Bullet(double speedx, double speedy, int rotation, Rocket rocket)
    {
        super(speedx, speedy);
        this.rocket = rocket;
        setRotation(rotation);
        increaseSpeed(15 * Math.cos(Math.toRadians(rotation)), 15 * Math.sin(Math.toRadians(rotation)));
        //Greenfoot.playSound("EnergyGun.wav");
    }
    
    /**
     * The bullet will damage asteroids if it hits them.
     */
    public void act()
    {
        if(life <= 0) {
            getWorld().removeObject(this);
        } else {
            moveAtSpeed();
            for (Asteroid asteroid : (List<Asteroid>)getWorld().getObjects(Asteroid.class))
            {
                if(touching(asteroid)){
                    getWorld().removeObject(this);
                    if (asteroid.hit(damage))
                    {
                        rocket.destroyedAsteroid();
                    }
                    return;
                }
            }
            life--;
        }
    }
    
    private boolean touching(Asteroid asteroid)
    {
        double distX = asteroid.getX() - getX();
        double distY = asteroid.getY() - getY();
        double dist = Math.sqrt(distX * distX + distY * distY);
        
        return (dist < asteroid.getRadius());
    }
}