import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.util.List;

/**
 * A rocket that can be controlled by the arrowkeys: up, left, right.
 * The gun is fired by hitting the 'space' key.
 * 
 * @author Poul Henriksen
 */
public class Rocket extends MovingThing
{
    private GreenfootImage rocket = new GreenfootImage("rocket.png");    
    private GreenfootImage rocketWithThrust = new GreenfootImage("rocketWithThrust.png");

    /** The minimum delay between firing the gun. */
    private int minGunFireDelay = 5;    
    /** How long ago we fired the gun the last time. */
    private int gunFireDelay = 0;    
    /** How fast the rocket is */  
    private double acceleration = 0.1;
    /** The player's score */
    private int score = 0;

    public Rocket()
    {
    }

    public void act()
    {
        if (checkCollision())
        {
            return;
        }

        ignite(Greenfoot.isKeyDown("up"));
        if(Greenfoot.isKeyDown("left")) {
            setRotation(getRotation() - 5);
        }        
        if(Greenfoot.isKeyDown("right")) {
            setRotation(getRotation() + 5);
        }

        if(Greenfoot.isKeyDown("space")) {
            if(gunFireDelay >= minGunFireDelay) {
                fireLaser();
                gunFireDelay = 0;
            }         
        }        
        gunFireDelay += 1;
        moveAtSpeed();
    }

    /**
     * Should the rocket be ignited?
     */
    private void ignite(boolean b) {
        if(b) {
            setImage(rocketWithThrust);
            accelerateDirection(getRotation(), acceleration);
        }
        else {
            setImage(rocket);            
        }
    }
    
    /**
     * Checks to see if we have collided with an asteroid
     */
    private boolean checkCollision()
    {
        Asteroid a = (Asteroid) getOneIntersectingObject(Asteroid.class);
        if(a != null)
        {
            World space = getWorld();
            space.removeObject(this);
            return true;
        }
        return false;
    }
    
    /**
     * Fires a bullet
     */
    private void fireBullet()
    {
         Bullet b = new Bullet(getSpeedX(), getSpeedY(), getRotation(), this);
         getWorld().addObject(b, getX(), getY());
         b.moveAtSpeed();
    }
    
    private void fireLaser()
    {
        double dirX = Math.cos(Math.toRadians(getRotation()));
        double dirY = Math.sin(Math.toRadians(getRotation()));
        
        for (Asteroid asteroid : (List<Asteroid>)getWorld().getObjects(Asteroid.class))
        {
            double distX = getX() - asteroid.getX();
            double distY = getY() - asteroid.getY();
            
            double a = dirX * dirX + dirY * dirY;
            double b = 2 * distX * dirX + 2 * distY * dirY;
            double c = distX * distX + distY * distY - asteroid.getRadius() * asteroid.getRadius();
            
            if (b * b - 4 * a * c >= 0)
            {
                if (asteroid.hit(30))
                {
                    destroyedAsteroid();
                }
            }
        }
        
        Laser.drawLaser(getWorld(), getX(), getY(), getX() + (int)(dirX * 800), getY() + (int)(dirY * 800));
    }
    
    public int getScore()
    {
        return score;
    }
    
    public void destroyedAsteroid()
    {
        score = score + 10;
    }
}